@@ -43,7 +43,9 @@ func NewClient(ctx context.Context, dialer N.Dialer, serverAddr M.Socksaddr, opt
 			},
 		}
 	} else {
-		tlsConfig.SetNextProtos([]string{http2.NextProtoTLS})
+		if len(tlsConfig.NextProtos()) == 0 {
+			tlsConfig.SetNextProtos([]string{http2.NextProtoTLS})
+		}
 		transport = &http2.Transport{
 			ReadIdleTimeout: time.Duration(options.IdleTimeout),
 			PingTimeout:     time.Duration(options.PingTimeout),
@@ -137,15 +139,16 @@ func (c *Client) dialHTTP2(ctx context.Context) (net.Conn, error) {
 	default:
 		request.Host = c.host[rand.Intn(hostLen)]
 	}
-	conn := newLateHTTPConn(pipeInWriter)
+	conn := NewLateHTTPConn(pipeInWriter)
 	go func() {
 		response, err := c.transport.RoundTrip(request)
 		if err != nil {
-			conn.setup(nil, err)
+			conn.Setup(nil, err)
 		} else if response.StatusCode != 200 {
-			conn.setup(nil, E.New("unexpected status: ", response.StatusCode, " ", response.Status))
+			response.Body.Close()
+			conn.Setup(nil, E.New("unexpected status: ", response.StatusCode, " ", response.Status))
 		} else {
-			conn.setup(response.Body, nil)
+			conn.Setup(response.Body, nil)
 		}
 	}()
 	return conn, nil
