@@ -13,7 +13,7 @@ import (
 	M "github.com/sagernet/sing/common/metadata"
 	N "github.com/sagernet/sing/common/network"
 
-	"github.com/gofrs/uuid"
+	"github.com/gofrs/uuid/v5"
 )
 
 type Client struct {
@@ -132,6 +132,10 @@ func (c *Conn) Write(b []byte) (n int, err error) {
 	return c.protocolConn.Write(b)
 }
 
+func (c *Conn) NeedAdditionalReadDeadline() bool {
+	return true
+}
+
 func (c *Conn) Upstream() any {
 	return c.Conn
 }
@@ -212,6 +216,10 @@ func (c *PacketConn) FrontHeadroom() int {
 	return 2
 }
 
+func (c *PacketConn) NeedAdditionalReadDeadline() bool {
+	return true
+}
+
 func (c *PacketConn) Upstream() any {
 	return c.Conn
 }
